@@ -6,13 +6,14 @@
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/timer.h>
+#include <osmocom/core/tdef.h>
 #include <osmocom/ctrl/control_if.h>
+#include <osmocom/gtp/gtp.h>
 
 #include "../lib/tun.h"
 #include "../lib/ippool.h"
 #include "../lib/syserr.h"
 #include "../lib/in46_addr.h"
-#include "../gtp/gtp.h"
 
 #include "sgsn.h"
 
@@ -65,9 +66,9 @@ struct apn_ctx {
 		uint32_t apn_type_mask;
 		/* GTP-U via TUN device or in Linux kernel */
 		enum apn_gtpu_mode gtpu_mode;
-		/* administratively shut-down (true) or not (false) */
+		/* administratively shut down (true) or not (false) */
 		bool shutdown;
-		/* transmit G-PDU sequeence numbers (true) or not (false) */
+		/* transmit G-PDU sequence numbers (true) or not (false) */
 		bool tx_gpdu_seq;
 	} cfg;
 
@@ -127,7 +128,7 @@ struct ggsn_ctx {
 		char *state_dir;
 		/* Time between Echo requests on each SGSN */
 		unsigned int echo_interval;
-		/* administratively shut-down (true) or not (false) */
+		/* administratively shut down (true) or not (false) */
 		bool shutdown;
 	} cfg;
 
@@ -152,6 +153,7 @@ struct apn_ctx *ggsn_find_or_create_apn(struct ggsn_ctx *ggsn, const char *name)
 /* ggsn_main.c */
 extern struct ctrl_handle *g_ctrlh;
 extern void *tall_ggsn_ctx;
+extern struct osmo_tdef_group ggsn_tdef_group[];
 
 /* ggsn.c */
 extern int ggsn_start(struct ggsn_ctx *ggsn);
