@@ -22,6 +22,8 @@
 #include <inttypes.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/utils.h>
@@ -35,13 +37,14 @@
 #include <osmocom/vty/misc.h>
 #include <osmocom/vty/tdef_vty.h>
 
-#include "../gtp/gtp.h"
-#include "../gtp/pdp.h"
+#include <osmocom/gtp/gtp.h>
+#include <osmocom/gtp/pdp.h>
 
 #include "../lib/util.h"
 
 #include "ggsn.h"
 #include "sgsn.h"
+#include "../gtp/gtp_internal.h"
 
 #define PREFIX_STR	"Prefix (Network/Netmask)\n"
 #define IFCONFIG_STR	"GGSN-based interface configuration\n"
@@ -223,6 +226,11 @@ DEFUN(cfg_ggsn_state_dir, cfg_ggsn_state_dir_cmd,
 {
 	struct ggsn_ctx *ggsn = (struct ggsn_ctx *) vty->index;
 
+	if (mkdir(argv[0], 0755) == -1 && errno != EEXIST) {
+		vty_out(vty, "%% Failed to create state-dir: %s%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
 	osmo_talloc_replace_string(ggsn, &ggsn->cfg.state_dir, argv[0]);
 
 	return CMD_SUCCESS;
