@@ -55,10 +55,13 @@
 
 /* #include <stdint.h>  ISO C99 types */
 
-#include "pdp.h"
-#include "gtp.h"
-#include "gtpie.h"
+#include <osmocom/gtp/pdp.h>
+#include <osmocom/gtp/gtp.h>
+#include <osmocom/gtp/gtpie.h>
+
 #include "queue.h"
+#include "gsn_internal.h"
+#include "gtp_internal.h"
 
 /* Error reporting functions */
 
@@ -638,8 +641,8 @@ int gtp_echo_req(struct gsn_t *gsn, int version, void *cbp,
 }
 
 /* Send off an echo reply */
-int gtp_echo_resp(struct gsn_t *gsn, int version,
-		  struct sockaddr_in *peer, int fd, void *pack, unsigned len)
+static int gtp_echo_resp(struct gsn_t *gsn, int version,
+			 struct sockaddr_in *peer, int fd, void *pack, unsigned len)
 {
 	union gtp_packet packet;
 	unsigned int length = get_default_gtp(version, GTP_ECHO_RSP, &packet);
@@ -650,8 +653,8 @@ int gtp_echo_resp(struct gsn_t *gsn, int version,
 }
 
 /* Handle a received echo request */
-int gtp_echo_ind(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
-		 int fd, void *pack, unsigned len)
+static int gtp_echo_ind(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
+			int fd, void *pack, unsigned len)
 {
 
 	/* Check if it was a duplicate request */
@@ -663,8 +666,8 @@ int gtp_echo_ind(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
 }
 
 /* Handle a received echo reply */
-int gtp_echo_conf(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
-		  void *pack, unsigned len)
+static int gtp_echo_conf(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
+			 void *pack, unsigned len)
 {
 	union gtpie_member *ie[GTPIE_SIZE];
 	unsigned char recovery;
@@ -716,8 +719,8 @@ int gtp_echo_conf(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
  * only listen to the GTP0 port, and therefore will never receive
  * anything else than GTP0 */
 
-int gtp_unsup_req(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
-		  int fd, void *pack, unsigned len)
+static int gtp_unsup_req(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
+			 int fd, void *pack, unsigned len)
 {
 	union gtp_packet packet;
 
@@ -727,8 +730,8 @@ int gtp_unsup_req(struct gsn_t *gsn, int version, struct sockaddr_in *peer,
 }
 
 /* Handle a Version Not Supported message */
-int gtp_unsup_ind(struct gsn_t *gsn, struct sockaddr_in *peer,
-		  void *pack, unsigned len)
+static int gtp_unsup_ind(struct gsn_t *gsn, struct sockaddr_in *peer,
+			 void *pack, unsigned len)
 {
 
 	if (gsn->cb_unsup_ind)
@@ -820,6 +823,40 @@ int gtp_ran_info_relay_req(struct gsn_t *gsn, const struct sockaddr_in *peer,
 	return gtp_notification(gsn, 1, &packet, length, peer, gsn->fd1c, 0);
 }
 
+/* ***********************************************************
+ * Conversion functions
+ *************************************************************/
+
+/* ***********************************************************
+ * IP address conversion functions
+ * There exist several types of address representations:
+ * - eua: End User Address. (29.060, 7.7.27, message type 128)
+ *   Used for signalling address to mobile station. Supports IPv4
+ *   IPv6 x.25 etc. etc.
+ * - gsna: GSN Address. (29.060, 7.7.32, message type 133): IP address
+ *   of GSN. If length is 4 it is IPv4. If length is 16 it is IPv6.
+ * - in_addr: IPv4 address struct.
+ * - sockaddr_in: Socket API representation of IP address and
+ *   port number.
+ *************************************************************/
+
+int gsna2in_addr(struct in_addr *dst, struct ul16_t *gsna)
+{
+	memset(dst, 0, sizeof(struct in_addr));
+	if (gsna->l != 4)
+		return EOF;	/* Return if not IPv4 */
+	memcpy(dst, gsna->v, gsna->l);
+	return 0;
+}
+
+static int in_addr2gsna(struct ul16_t *gsna, struct in_addr *src)
+{
+	memset(gsna, 0, sizeof(struct ul16_t));
+	gsna->l = 4;
+	memcpy(gsna->v, src, gsna->l);
+	return 0;
+}
+
 /* ***********************************************************
  * Session management messages
  * Messages: create, update and delete PDP context
@@ -988,22 +1025,9 @@ int gtp_create_context_req(struct gsn_t *gsn, struct pdp_t *pdp,
 	return 0;
 }
 
-/* API: Application response to context indication */
-int gtp_create_context_resp(struct gsn_t *gsn, struct pdp_t *pdp, int cause)
-{
-
-	/* Now send off a reply to the peer */
-	gtp_create_pdp_resp(gsn, pdp->version, pdp, cause);
-
-	if (!gtp_cause_successful(cause))
-		gtp_freepdp(gsn, pdp);
-
-	return 0;
-}
-
 /* Send Create PDP Context Response */
-int gtp_create_pdp_resp(struct gsn_t *gsn, int version, struct pdp_t *pdp,
-			uint8_t cause)
+static int gtp_create_pdp_resp(struct gsn_t *gsn, int version, struct pdp_t *pdp,
+			       uint8_t cause)
 {
 	union gtp_packet packet;
 	unsigned int length =
@@ -1064,10 +1088,23 @@ int gtp_create_pdp_resp(struct gsn_t *gsn, int version, struct pdp_t *pdp,
 			pdp->fd, pdp->seq, pdp->tid);
 }
 
+/* API: Application response to context indication */
+int gtp_create_context_resp(struct gsn_t *gsn, struct pdp_t *pdp, int cause)
+{
+
+	/* Now send off a reply to the peer */
+	gtp_create_pdp_resp(gsn, pdp->version, pdp, cause);
+
+	if (!gtp_cause_successful(cause))
+		gtp_freepdp(gsn, pdp);
+
+	return 0;
+}
+
 /* Handle Create PDP Context Request */
-int gtp_create_pdp_ind(struct gsn_t *gsn, int version,
-		       struct sockaddr_in *peer, int fd,
-		       void *pack, unsigned len)
+static int gtp_create_pdp_ind(struct gsn_t *gsn, int version,
+			      struct sockaddr_in *peer, int fd,
+			      void *pack, unsigned len)
 {
 	struct pdp_t *pdp, *pdp_old;
 	struct pdp_t pdp_buf;
@@ -1390,8 +1427,8 @@ recover_ret:
 }
 
 /* Handle Create PDP Context Response */
-int gtp_create_pdp_conf(struct gsn_t *gsn, int version,
-			struct sockaddr_in *peer, void *pack, unsigned len)
+static int gtp_create_pdp_conf(struct gsn_t *gsn, int version,
+			       struct sockaddr_in *peer, void *pack, unsigned len)
 {
 	struct pdp_t *pdp;
 	union gtpie_member *ie[GTPIE_SIZE];
@@ -2157,11 +2194,11 @@ int gtp_delete_context_req2(struct gsn_t *gsn, struct pdp_t *pdp, void *cbp,
 }
 
 /* Send Delete PDP Context Response */
-int gtp_delete_pdp_resp(struct gsn_t *gsn, int version,
-			struct sockaddr_in *peer, int fd,
-			void *pack, unsigned len,
-			struct pdp_t *pdp, struct pdp_t *linked_pdp,
-			uint8_t cause, int teardown)
+static int gtp_delete_pdp_resp(struct gsn_t *gsn, int version,
+			       struct sockaddr_in *peer, int fd,
+			       void *pack, unsigned len,
+			       struct pdp_t *pdp, struct pdp_t *linked_pdp,
+			       uint8_t cause, int teardown)
 {
 	union gtp_packet packet;
 	unsigned int length =
@@ -2202,9 +2239,9 @@ int gtp_delete_pdp_resp(struct gsn_t *gsn, int version,
 }
 
 /* Handle Delete PDP Context Request */
-int gtp_delete_pdp_ind(struct gsn_t *gsn, int version,
-		       struct sockaddr_in *peer, int fd,
-		       void *pack, unsigned len)
+static int gtp_delete_pdp_ind(struct gsn_t *gsn, int version,
+			      struct sockaddr_in *peer, int fd,
+			      void *pack, unsigned len)
 {
 	struct pdp_t *pdp = NULL;
 	struct pdp_t *linked_pdp = NULL;
@@ -2301,8 +2338,8 @@ int gtp_delete_pdp_ind(struct gsn_t *gsn, int version,
 }
 
 /* Handle Delete PDP Context Response */
-int gtp_delete_pdp_conf(struct gsn_t *gsn, int version,
-			struct sockaddr_in *peer, void *pack, unsigned len)
+static int gtp_delete_pdp_conf(struct gsn_t *gsn, int version,
+			       struct sockaddr_in *peer, void *pack, unsigned len)
 {
 	union gtpie_member *ie[GTPIE_SIZE];
 	uint8_t cause;
@@ -3050,61 +3087,6 @@ int gtp_data_req(struct gsn_t *gsn, struct pdp_t *pdp, void *pack, unsigned len)
 	return 0;
 }
 
-/* ***********************************************************
- * Conversion functions
- *************************************************************/
-
-/* ***********************************************************
- * IP address conversion functions
- * There exist several types of address representations:
- * - eua: End User Address. (29.060, 7.7.27, message type 128)
- *   Used for signalling address to mobile station. Supports IPv4
- *   IPv6 x.25 etc. etc.
- * - gsna: GSN Address. (29.060, 7.7.32, message type 133): IP address
- *   of GSN. If length is 4 it is IPv4. If length is 16 it is IPv6.
- * - in_addr: IPv4 address struct.
- * - sockaddr_in: Socket API representation of IP address and
- *   port number.
- *************************************************************/
-
-int ipv42eua(struct ul66_t *eua, struct in_addr *src)
-{
-	eua->v[0] = PDP_EUA_ORG_IETF;
-	eua->v[1] = PDP_EUA_TYPE_v4;
-	if (src) {
-		eua->l = 6;
-		memcpy(&eua->v[2], src, 4);
-	} else {
-		eua->l = 2;
-	}
-	return 0;
-}
-
-int eua2ipv4(struct in_addr *dst, struct ul66_t *eua)
-{
-	if ((eua->l != 6) || (eua->v[0] != PDP_EUA_ORG_IETF) || (eua->v[1] != PDP_EUA_TYPE_v4))
-		return -1;	/* Not IPv4 address */
-	memcpy(dst, &eua->v[2], 4);
-	return 0;
-}
-
-int gsna2in_addr(struct in_addr *dst, struct ul16_t *gsna)
-{
-	memset(dst, 0, sizeof(struct in_addr));
-	if (gsna->l != 4)
-		return EOF;	/* Return if not IPv4 */
-	memcpy(dst, gsna->v, gsna->l);
-	return 0;
-}
-
-int in_addr2gsna(struct ul16_t *gsna, struct in_addr *src)
-{
-	memset(gsna, 0, sizeof(struct ul16_t));
-	gsna->l = 4;
-	memcpy(gsna->v, src, gsna->l);
-	return 0;
-}
-
 /* TS 29.060 has yet again a different encoding for IMSIs than
  * what we have in other places, so we cannot use the gsm48
  * decoding functions.  Also, libgtp uses an uint64_t in
